var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var ActionTrigger_exports = {};
__export(ActionTrigger_exports, {
  default: () => ActionTrigger_default
});
module.exports = __toCommonJS(ActionTrigger_exports);
var import_lodash = require("lodash");
var import_database = require("@nocobase/database");
var import_plugin_workflow = require("@nocobase/plugin-workflow");
var import_data_source_manager = require("@nocobase/data-source-manager");
class ActionTrigger_default extends import_plugin_workflow.Trigger {
  constructor(workflow) {
    super(workflow);
    workflow.app.use(this.middleware, { after: "dataSource" });
  }
  async triggerAction(context, next) {
    const { triggerWorkflows } = context.action.params;
    if (!triggerWorkflows) {
      return context.throw(400);
    }
    context.status = 202;
    await next();
    this.trigger(context);
  }
  middleware = async (context, next) => {
    const {
      resourceName,
      actionName,
      params: { triggerWorkflows }
    } = context.action;
    if (resourceName === "workflows" && actionName === "trigger") {
      return this.triggerAction(context, next);
    }
    await next();
    if (!triggerWorkflows) {
      return;
    }
    if (!["create", "update"].includes(actionName)) {
      return;
    }
    return this.trigger(context);
  };
  async trigger(context) {
    const { triggerWorkflows = "", values } = context.action.params;
    const dataSourceHeader = context.get("x-data-source") || "main";
    const { currentUser, currentRole } = context.state;
    const { model: UserModel } = this.workflow.db.getCollection("users");
    const userInfo = {
      user: UserModel.build(currentUser).desensitize(),
      roleName: currentRole
    };
    const triggers = triggerWorkflows.split(",").map((trigger) => trigger.split("!"));
    const workflowRepo = this.workflow.db.getRepository("workflows");
    const workflows = (await workflowRepo.find({
      filter: {
        key: triggers.map((trigger) => trigger[0]),
        current: true,
        type: "action",
        enabled: true
      }
    })).filter((workflow) => Boolean(workflow.config.collection));
    const syncGroup = [];
    const asyncGroup = [];
    for (const workflow of workflows) {
      const { collection, appends = [] } = workflow.config;
      const [dataSourceName, collectionName] = (0, import_data_source_manager.parseCollectionName)(collection);
      const trigger = triggers.find((trigger2) => trigger2[0] == workflow.key);
      const event = [workflow];
      if (context.action.resourceName !== "workflows") {
        if (!context.body) {
          continue;
        }
        if (dataSourceName !== dataSourceHeader) {
          continue;
        }
        const { body: data } = context;
        for (const row of Array.isArray(data) ? data : [data]) {
          let payload = row;
          if (trigger[1]) {
            const paths = trigger[1].split(".");
            for (const field of paths) {
              if (payload.get(field)) {
                payload = payload.get(field);
              } else {
                const association = (0, import_database.modelAssociationByKey)(payload, field);
                payload = await payload[association.accessors.get]();
              }
            }
          }
          const model = payload.constructor;
          if (payload instanceof import_database.Model) {
            if (collectionName !== model.collection.name) {
              continue;
            }
            if (appends.length) {
              payload = await model.collection.repository.findOne({
                filterByTk: payload.get(model.primaryKeyAttribute),
                appends
              });
            }
          }
          event.push({ data: (0, import_plugin_workflow.toJSON)(payload), ...userInfo });
        }
      } else {
        const { model, repository } = context.app.dataSourceManager.dataSources.get(dataSourceName).collectionManager.getCollection(collectionName);
        let data = trigger[1] ? (0, import_lodash.get)(values, trigger[1]) : values;
        const pk = (0, import_lodash.get)(data, model.primaryKeyAttribute);
        if (appends.length && pk != null) {
          data = await repository.findOne({
            filterByTk: pk,
            appends
          });
        }
        event.push({ data, ...userInfo });
      }
      (workflow.sync ? syncGroup : asyncGroup).push(event);
    }
    for (const event of syncGroup) {
      await this.workflow.trigger(event[0], event[1], { httpContext: context });
    }
    for (const event of asyncGroup) {
      this.workflow.trigger(event[0], event[1]);
    }
  }
  on(workflow) {
  }
  off(workflow) {
  }
}
